module coeffs
!--------------------------------------------------------------------------------------
! Coefficient and boundary condition functions that define the general 2D linear PDE:
!
!     au_xx + bu_yy + cu_x + du_y + eu = f,
!     subject to u = BC on the boundaries
!
! The boundary conditions are Dirichlet. All of a-f can be functions of x and y. The
! values used here correspond to an explicit solution function  as defined in BC().
! So the solution should be the same as the boundary conditions ... unless I have
! miscalculated. Which is unpossible.
!-----------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!--------------------------------------------------------------------------------------
    use kindlytypes, only: real8
    real(kind=real8), parameter :: pi = 3.1415926535897924_real8

contains 
    real(kind=real8) function afun(x,y) 
        real(kind=real8) :: x, y
        afun =  1.0_real8 + y
        return
    end function afun

    real(kind=real8) function  bfun(x,y) 
        real(kind=real8) :: x, y
        bfun = 1.0_real8 + x
        return
    end function bfun

    real(kind=real8) function  cfun(x,y) 
        real(kind=real8) :: x, y
        cfun = 1.0_real8
        return
    end function cfun

    real(kind=real8) function  dfun(x,y) 
        real(kind=real8) :: x, y
        dfun =  -2.0_real8
        return
    end function dfun

    real(kind=real8) function  efun(x,y) 
        real(kind=real8) :: x, y
        efun = -x
        return
    end function efun

    real(kind=real8) function  ffun(x,y) 
        real(kind=real8) :: x, y
        ffun = -x**4 + 3.0_real8*x**2 + (8.0_real8 + 6.0_real8*y - y**2)*x &
                - 4.0_real8*y + 2.0_real8
        return
    end function ffun

    real(kind=real8) function BC(x,y) 
        real(kind=real8) :: x, y
        BC = x**3 + y**2
        return
    end function BC
        
end module coeffs
