module coeffs
!--------------------------------------------------------------------------------------
! Coefficient and boundary condition functions that define the general 2D linear PDE:
!
!     au_xx + bu_yy + cu_x + du_y + eu = f,         (1)
!     subject to u = BC on the boundaries           (2)
! 
! The solution u(x,y) is actually given by the BC function. I started from that,
! made up a-e, and then computed f to equal the left hand side of (1). See the
! Matlab script exacto.m for the actual computations of the functions.
!
!-----------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!--------------------------------------------------------------------------------------
    use kindlytypes, only: real8
    real(kind=real8), parameter :: pi = 3.1415926535897924_real8

contains 
    real(kind=real8) function afun(x,y) 
        real(kind=real8) :: x, y
        afun =  exp(x - y)
        return
    end function afun

    real(kind=real8) function  bfun(x,y) 
        real(kind=real8) :: x, y
        bfun = asinh(x + y + 1.0_real8)
        return
    end function bfun

    real(kind=real8) function  cfun(x,y) 
        real(kind=real8) :: x, y
        cfun = pi*y
        return
    end function cfun

    real(kind=real8) function  dfun(x,y) 
        real(kind=real8) :: x, y
        dfun = x*y
        return
    end function dfun

    real(kind=real8) function  efun(x,y) 
        real(kind=real8) :: x, y
        efun = 1.0_real8
        return
    end function efun

    real(kind=real8) function  ffun(x,y) 
        real(kind=real8) :: x, y, s
        s = x**2 + y**2 + 2.0_real8
        ffun = sin(x + y) - cos(x - y) + exp(x - y)*(cos(x - y) - sin(x + y) + &
               (8.0*x**2)/s**3 - 2.0_real8/s**2) + 1.0_real8/s + asinh(x + y + &
               1.0_real8)*(cos(x - y) - sin(x + y) + (8.0_real8*y**2)/s**3 -   &
               2.0_real8/s**2) + y*pi*(sin(x - y) + cos(x + y) - &
               (2.0_real8*x)/s**2) - x*y*(sin(x - y) - cos(x + y) &
                + (2.0_real8*y)/s**2)
        return
    end function ffun

    real(kind=real8) function BC(x,y) 
        real(kind=real8) :: x, y
        BC = sin(x + y) - cos(x - y) + 1/(x**2 + y**2 + 2.0_real8)
        return
    end function BC
        
end module coeffs
