#-----------------------------------------------------------------------------

# Prevents attempts to regard all .mod files as modula-2 and using m2c to compile
# them into .c files:
%.o : %.mod 


#===== Compilers =================================================================
CC=gcc

# F95=ifort
F95=gfortran

#===== Optimizations =============================================================
# Some of these are tied to the compiler being Intel. Big fat hairy deal, heh?

# Turned off -Wall because of too many warnings about unused variables.
ifeq "$(strip $(F95))" "gfortran"
	OPTS = \
	-O3 \
	-std=f2008 \
	-pedantic \
	-fno-range-check		# to allow setting NaN, +/-Inf

	#     -ansi  \
	#     -pedantic  \
	#     -Werror  \
	#     -Wall  \
	#     -W  \
	#     -Wmissing-prototypes  \
	#     -Wstrict-prototypes  \
	#     -Wconversion  \
	#     -Wshadow  \
	#     -Wpointer-arith  \
	#     -Wcast-qual  \
	#     -Wcast-align  \
	#     -Wwrite-strings  \
	#     -Wnested-externs  \
	#     -fshort-enums  \
	#     -fno-common  \
	#     -Dinline=  \
	#     -g  \

else
	OPTS = -O3     \
    # -opt-prefetch   \
    # -align all      \
    # -ccdefault none \
    # -ftz            \
    # -funroll-loops  \
    # -pad            \
    # -falign-functions=16 \
    # -fp-model fast=2 \
    # -fp-speculation=fast \
    # -opt-prefetch \
    # -no-prec-div \
    # -xHost \

	# -integer-size 64 \
	# -traceback -warn general\
	# -warn all \
	# -openmp 	\				  # For OpenMP-using programs
	# -check arg_temp_created \   # Can be much slower!

endif

CCOPTS = -O3

# use -xAVX in behemoth, or other systems where the cpu supports
# it. I.e., AVX > SSE4.2. When compiling on the machine to execute
# the code, -xHost should take care of that.


#===== Libraries and Other Effluvia ==============================================

# Use the 2013 Intel libs:
MKLROOT = /opt/intel/composer_xe_2013.3.163/mkl
LIBS =   -L$(MKLROOT)/lib/intel64 -lmkl_rt -lpthread -lm
# add -lgomp for OpenMP

#------------------------------------------------------
# Other variants, from older intel libs
#------------------------------------------------------
#
# LIBS = -L/usr/lib/x86_64-linux-gnu/ \
#        -L/opt/intel/Compiler/latest/mkl/lib/em64t \
#        -lmkl_em64t -lguide -lpthread
# 
# LIBS = -L/opt/intel/Compiler/latest/mkl/lib/em64t \
#        -L/opt/intel/Compiler/latest/lib/intel64 \
#        -lmkl_em64t -lguide 
#
#---------------------------------------
# For GNU gfortran and GSL: 
# LIBS = -L/usr/lib/ -lgsl -lblas
