
subroutine timestamp(outunit, date, time, zone)
!--------------------------------------------------------------------------------------
! Attach the date and time to the I/O unit outunit. The unit is assumed to be already
! opened. The world ends if it is not. Because  date and time are optional arguments,
! an explicit interface must be used in whoever calls this routine. The format
! statement prepends a % before the values, so that the line becomes a Matlab comment 
! line. Change to your favorite as desired.
!
! This also returns the date and time if you want to masticate on it yourself.
!------------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!--------------------------------------------------------------------------------------

    use kindlytypes, only: int4 => integer4
    use iso_fortran_env, only: stdout => output_unit, &
                               stderr => error_unit
    implicit none
    character(len=15), optional, intent(inout) :: date, time, zone
    character(len=15) :: tsdate, tstime, tszone
    integer(kind=int4), intent(in) :: outunit
    integer :: time_array(8)

    call date_and_time (date=tsdate, time=tstime, zone=tszone, values=time_array)
!    current_time = time_array(8) * (1.0d-3)    &   ! number of milliseconds
!                 + time_array(7) * (1.0d0)     &   ! number of seconds
!                 + time_array(6) * (6.0d1)     &   ! number of minutes
!                 + time_array(5) * (3.6d3)     &   ! number of hours
!                 + time_array(3) * (24*3.6d3)  ! number of days

    write(outunit, 100) tsdate(7:8), tsdate(5:6), tsdate(1:4), &
                        tstime(1:2), tstime(3:4), tstime(5:10)
    100 format(//,'% Date: ',a2,'/',a2,'/',a4,'  Time: ',a2,':',a2,':',a6,//)

    if (present(date)) then
        date = tsdate
    end if

    if (present(time)) then
        time = tstime
    end if

    if (present(zone)) then
        zone = tszone
    end if

    return

end subroutine timestamp
